@@ -105,6 +105,8 @@
 #' @param sigma Numeric value to use for subsequent predictions or
 #'   back-transformation bias adjustments. If not specified, we use
 #'   \code{sigma(object)}, if available, and \code{NULL} otherwise.
+#'   Note: This applies only when the family is \code{"gaussian"}; for other families,
+#'   \code{sigma} is set to \code{NA} and cannot be overridden.
 #' @param counterfactuals,wt.counter,avg.counter \code{counterfactuals} specifies character
 #'   names of counterfactual factors. If this is non-missing, a reference grid
 #'   is created consisting of combinations of counterfactual levels and a constructed
@@ -453,19 +455,14 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
         options$df = df
     }
     
-    if(missing(sigma)) { # Get 'sigma(object)' if available, else NULL
-        sigma = suppressWarnings(try(stats::sigma(object), silent = TRUE))
-        if (inherits(sigma, "try-error"))
-            sigma = NULL
-    }
-    
     # recover the data
     if (missing(data)) {
         data = try(recover_data (object, data = NULL, ...))
         if (inherits(data, "try-error"))
             stop("Perhaps a 'data' or 'params' argument is needed")
     }
-    else # attach needed attributes to given data
+    # note if options$delts non-null, we called 2nd time from emtrends and data is ok as-is
+    else if (is.null(options$delts)) # attach needed attributes to given data
         data = recover_data(object, data = as.data.frame(data), ...)
     
     if(is.character(data)) # 'data' is in fact an error message
@@ -820,17 +817,25 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
 
     # Any offsets??? (misc$offset.mult might specify removing or reversing the offset)
     om = ifelse(is.null(misc$offset.mult), 1, misc$offset.mult)
+    oval = 0
     if (!missing(offset)) {  # For safety, we always treat it as scalar
         if (offset[1] != 0)
-            grid[[".offset."]] = offset[1]
+            oval = offset[1]
     }
-    else if(".offset." %in% names(grid))
-        grid[[".offset."]] = om * grid[[".offset."]]
-    else if(!is.null(attr(trms,"offset"))) {
-        if (any(om != 0))
-            grid[[".offset."]] = om * .get.offset(trms, grid)
+    else {
+        if(".static.offset." %in% names(grid)) { # static offset is available
+            oval = om * grid[[".static.offset."]]
+            # grid$.static.offset. = NULL
+        }
+        #else 
+        if(!is.null(attr(trms,"offset"))) {
+            if (any(om != 0))
+                oval = om * (oval + .get.offset(trms, grid))
+        }
+        if(any(oval != 0))
+            grid[[".offset."]] = oval
     }
-
+    
     ### --- Determine weights for each grid point
     if (!hasName(data, "(weights)"))
         data[["(weights)"]] = 1
@@ -876,7 +881,18 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
     misc$famSize = nrow(grid)
     if(is.null(misc$avgd.over))
        misc$avgd.over = character(0)
-    misc$sigma = sigma
+    
+    # emm_basis may have provided a sigma. If not, use sigma value from model or args
+    if(is.null(misc$sigma) && missing(sigma)) { # Get 'sigma(object)' if available, else NULL
+        sigma = suppressWarnings(try(stats::sigma(object), silent = TRUE))
+        if (inherits(sigma, "try-error"))
+            sigma = NULL
+        misc$sigma = sigma
+    }
+    # ... but override it only if sigma != NA
+    if(is.null(misc$sigma) || (length(misc$sigma) == 0) || !is.na(misc$sigma[1]))
+        misc$sigma = sigma
+    
 
     post.beta = basis$post.beta
     if (is.null(post.beta))
